Imports System.ComponentModel
Imports System.Text.RegularExpressions

<ToolboxBitmap(GetType(RegexProvider), "regex_provider.bmp"), _
 ProvideProperty("RegexPattern", GetType(TextBox)), _
 ProvideProperty("RegexMessage", GetType(TextBox))> _
Public Class RegexProvider
    Inherits System.ComponentModel.Component
    Implements IExtenderProvider

#Region " Component Designer generated code "

    Public Sub New(ByVal Container As System.ComponentModel.IContainer)
        MyClass.New()

        'Required for Windows.Forms Class Composition Designer support
        Container.Add(Me)
    End Sub

    Public Sub New()
        MyBase.New()

        'This call is required by the Component Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Component overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Component Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Component Designer
    'It can be modified using the Component Designer.
    'Do not modify it using the code editor.
    Friend WithEvents errBadMatch As System.Windows.Forms.ErrorProvider
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.errBadMatch = New System.Windows.Forms.ErrorProvider
        '
        'errBadMatch
        '
        Me.errBadMatch.BlinkStyle = System.Windows.Forms.ErrorBlinkStyle.NeverBlink

    End Sub

#End Region

    ' Information about a control's range.
    Private Class RegexInfo
        Public RegexPattern As String
        Public RegexMessage As String

        Public Sub New()
            RegexPattern = ""
            RegexMessage = ""
        End Sub

        ' Return True if this object represents no range.
        Public Function IsDefault() As Boolean
            Return (RegexPattern.Length = 0) AndAlso _
                   (RegexMessage.Length = 0)
        End Function
    End Class

    ' The information about fields with ranges.
    Private m_RegexInfo As New Hashtable

    Private m_Enabled As Boolean = True
    <Category("Behavior"), _
     DefaultValue(True)> _
    Public Property Enabled() As Boolean
        Get
            Return m_Enabled
        End Get
        Set(ByVal Value As Boolean)
            m_Enabled = Value
        End Set
    End Property

    ' We can extend TextBoxes.
    Public Function CanExtend(ByVal client_control As Object) As Boolean Implements IExtenderProvider.CanExtend
        Return (TypeOf client_control Is TextBox)
    End Function

    ' Return this control's error message.
    <Category("Validation"), _
     DefaultValue("")> _
    Public Function GetRegexPattern(ByVal client_control As TextBox) As String
        Return GetRegexInfo(client_control).RegexPattern
    End Function

    ' Set this control's error message.
    <Category("Validation"), _
     DefaultValue("")> _
    Public Sub SetRegexPattern(ByVal client_control As TextBox, ByVal error_message As String)
        ' Get this control's range info.
        Dim regex_info As RegexInfo = GetRegexInfo(client_control)

        ' Set the new minimum value.
        regex_info.RegexPattern = error_message

        ' Add or remove the RegexInfo if necessary.
        AddOrRemoveIfNecessary(client_control, regex_info)
    End Sub

    ' Return this control's error message.
    <Category("Validation"), _
     DefaultValue("")> _
    Public Function GetRegexMessage(ByVal client_control As TextBox) As String
        Return GetRegexInfo(client_control).RegexMessage
    End Function

    ' Set this control's error message.
    <Category("Validation"), _
     DefaultValue("")> _
    Public Sub SetRegexMessage(ByVal client_control As TextBox, ByVal error_message As String)
        ' Get this control's range info.
        Dim regex_info As RegexInfo = GetRegexInfo(client_control)

        ' Set the new minimum value.
        regex_info.RegexMessage = error_message

        ' Add or remove the RegexInfo if necessary.
        AddOrRemoveIfNecessary(client_control, regex_info)
    End Sub

    ' A client is validating. See if it is blank.
    Private Sub Client_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs)
        If Not m_Enabled Then Exit Sub
        ValidateTextBox(DirectCast(sender, TextBox))
    End Sub

    ' Validate the TextBox.
    Private Sub ValidateTextBox(ByVal client_control As TextBox)
        ' Get the control's text.
        Dim client_text As String = client_control.Text
        Dim regex_info As RegexInfo = GetRegexInfo(client_control)

        ' See if the text satisfies the regular expression.
        If Regex.IsMatch(client_text, regex_info.RegexPattern) Then
            ' It matches.
            errBadMatch.SetError(client_control, "")
        Else
            ' It doesn't match.
            errBadMatch.SetError(client_control, regex_info.RegexMessage)
        End If
    End Sub

    ' Return this control's RegexInfo.
    Private Function GetRegexInfo(ByVal client_control As Control) As RegexInfo
        ' See if we have RegexInfo for this control.
        If m_RegexInfo.Contains(client_control) Then
            ' We have RegexInfo for this control. Return it.
            Return DirectCast(m_RegexInfo(client_control), RegexInfo)
        Else
            ' We do not have RegexInfo for this control.
            ' Return a new default RegexInfo.
            Return New RegexInfo
        End If
    End Function

    ' Add or remove this RegexInfo if necessary.
    Private Sub AddOrRemoveIfNecessary(ByVal client_control As Control, ByVal regex_info As RegexInfo)
        ' See if the RegexInfo should be present but is not,
        ' or should not be present but is.
        If regex_info.IsDefault <> Not m_RegexInfo.Contains(client_control) Then
            If regex_info.IsDefault Then
                ' The RegexInfo should not be present but is.
                m_RegexInfo.Remove(client_control)
                RemoveHandler client_control.Validating, AddressOf Client_Validating
            Else
                ' The RegexInfo should be present but is not.
                m_RegexInfo.Add(client_control, regex_info)
                AddHandler client_control.Validating, AddressOf Client_Validating
            End If
        End If
    End Sub

    ' If some control has an error, display its error message,
    ' set focus to it, and return True. If all controls are
    ' okay, then return False.
    Public Function HasError() As Boolean
        If Not m_Enabled Then Return False

        ' Make sure all controls have been validated.
        Dim first_text_box As TextBox = Nothing
        Dim first_tab_index As Integer = Integer.MaxValue
        For Each text_box As TextBox In m_RegexInfo.Keys
            ' Validate the TextBox.
            ValidateTextBox(text_box)

            ' See if this is the first in the tab order so far.
            If errBadMatch.GetError(text_box).Length > 0 Then
                If text_box.TabIndex < first_tab_index Then
                    first_text_box = text_box
                    first_tab_index = text_box.TabIndex
                End If
            End If
        Next text_box

        ' See if any control has an error.
        If Not (first_text_box Is Nothing) Then
            first_text_box.Focus()
            MessageBox.Show(errBadMatch.GetError(first_text_box))
            Return True
        End If

        Return False
    End Function
End Class
